/*
 * main.cc -- Top-level functions.
 *
 * $Id: main.cc,v 1.8 2000/05/01 17:14:32 pedwards Exp $
*/

#include <sincedecls.h>
#include <stdio.h>
#include <string.h>
#include <opts.h>
#include <iostream>
#include <fstream>
#include <stdexcept>
#include <time.h>

string   dbname;
DBM*     db = NULL;
time_t   rightnow;
string   pagername;

namespace {
    char* tfilename = NULL;

    inline void displayfile (const string& f)
    {
        if (genopt_page_option) {
            system (((pagername + " ") + f).c_str());
        } else {
            // probably win a Useless Use Of Cat award here
            system ((("cat ") + f).c_str());
        }
    }

    void cleanup()
    {
        if (db)  dbm_close(db);
        if (tfilename && (remove(tfilename) != 0)) {
            fatal_error("could not remove temporary file ")
                << string(tfilename) << ":  " << strerror(errno);
        }
    }
};


int main (int argc, char *argv[])
{
    int retval = 0;

    if (atexit(cleanup) != 0)
        fatal_error("could not register cleanup functions");

    try {
        int nextarg = genopt_process_options (argc, argv);

        // find the database
        if (genopt_dbase_option_given) {
            dbname = genopt_dbase_option;
        } else {
            char *p = getenv("SINCE_DB");
            if (p)  dbname = p;
            else  fatal_error("SINCE_DB undefined and no -f option given");
        }

        char *pager = getenv("PAGER");
        if (pager)  pagername = pager;
        else  pagername = "/usr/bin/more";

        rightnow = time(NULL);
        if (genopt_reinit_option || genopt_reinitnow_option) {
            // just reset the times
            db = dbm_open (dbname.c_str(), O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
            if (db == NULL)
                fatal_error("could not open/create database ")
                    << dbname << ":  "
                    << strerror(errno);
            else {
                while (nextarg < argc) {
                    if (genopt_reinitnow_option)
                        reset_logfile (argv[nextarg++], rightnow);
                    else
                        reset_logfile (argv[nextarg++], 0);
                }
            }
        } else {
            // actually doing some work
            if (genopt_check_option || genopt_readonly_option) {
                db = dbm_open (dbname.c_str(), O_RDONLY, 0);
            } else {
                db = dbm_open (dbname.c_str(), O_RDWR, S_IRUSR | S_IWUSR);
            }
            if (db == NULL)  fatal_error("could not open database ")
                                   << dbname << ":  " << strerror(errno);

            // get a temporary file to accumulate text
            tfilename = tempnam (NULL, "llog");
            if (tfilename == NULL) {
                fatal_error("could not create temporary filename:  ")
                    << string(strerror(errno));
            }
            std::ofstream  tfile (tfilename);
            if (!tfile) {
                fatal_error("could not create temporary file ")
                    << string(tfilename) << ":  " << strerror(errno);
            }

            // fire off the functions for each logfile
            while (nextarg < argc) {
                if (genopt_check_option) {
                    retval += check_for_new (argv[nextarg]);
                } else {
                    append_new (argv[nextarg], tfile);
                    if (!genopt_readonly_option)
                        reset_logfile (argv[nextarg], rightnow);
                }
                ++nextarg;
            }

            tfile.close();
            if (getsize(tfilename) > 0)  displayfile(tfilename);
        }
    }
    catch (std::exception& e) {
        std::cerr << "Wow, something REALLY screwed up!  Exception:  "
                  << e.what() << std::endl;
        ::exit(1);
    }

    return retval;
}

