/*
 * work.cc -- Actually does the work of grinding through the database,
 *            appending new text to the temp file, and updating the database.
 *
 * $Id: work.cc,v 1.5 2000/05/01 17:06:06 pedwards Exp $
*/

#include <sincedecls.h>
#include <opts.h>
#include <string.h>
#include <stdio.h>
#include <fstream>
#include <unistd.h>

#define HAVE_MMAP
#include "Mmap.h"     // who knows where this has been installed...


namespace {
    // an entry is:   <last mod time> <file size at last mod time>
    const size_t  thingsize  = sizeof(time_t) + sizeof(size_t);
    const size_t  sizeindex  = sizeof(time_t);

    datum  mkentry (time_t t, size_t s)
    {
        unsigned char   dat[thingsize];

        memcpy (dat, &t, sizeof(time_t));
        memcpy (dat + sizeindex, &s, sizeof(size_t));

        datum e = {&dat, thingsize};
        return e;
    }

    inline void breakentry (datum d, time_t& t, size_t& s)
    {
        memcpy (&t, d.dptr, sizeof(time_t));
        memcpy (&s, (char*)(d.dptr)+sizeindex, sizeof(size_t));
    }
};


// The entry for the logfile named by f is brought up to time named by t.
void reset_logfile (const string& f, time_t t)
{
    datum  key = {const_cast<char*>(f.c_str()), f.length()};
    datum  entry = mkentry(t, getsize(f));

    if (dbm_store (db, key, entry, DBM_REPLACE) != 0)
        fatal_error("could not replace entry for ") << f;
}


// If the logfile named by f has new information, returns 1, else 0.  No
// other values currently make much sense.
int check_for_new (const string& f)
{
    // Beware that the returns from this function are being accumulated, so
    // we'll avoid the implicit bool->int conversion values for now, just
    // to be safe.
    return has_changed(f) ? 1 : 0;
}


// Append any new stuff for file f onto tfile (which is open).  There may
// be nothing to do, or only some new lines, or (if the file has been
// reset) the entire file, etc.
void append_new (const string& f, std::ofstream& tfile)
{
    size_t  lastsize;
    time_t  lastmod = getmtime(f),
            lastexamined;
    datum   key = {const_cast<char*>(f.c_str()), f.length()};
    datum   entry = dbm_fetch (db, key);

    if (entry.dptr == NULL) {
        // we could warn and continue on, I suppose...
        std::cerr << "Warning:  no entry for " << f << " in database.\nUse "
                  << "the '-i' option to initialize an entry." << std::endl;
    } else {
        breakentry (entry, lastexamined, lastsize);
        if (lastmod >= lastexamined) {
            // there have been changes
            size_t  nowsize = getsize(f);
            if (nowsize == 0)  return;  // mmap()'ing a zero-length file is bad
            Mmap m (f, PROT_READ);
            if (nowsize > lastsize) {
                // the file appears to have grown
                tfile.write (static_cast<char*>(m.srcp) + lastsize,
                          nowsize - lastsize);
            } else {
                // either the file has shrunk, or it has somehow been
                // "touched" without altering the size -- assume that it
                // got reset and that everything in the file is new
                tfile.write (static_cast<char*>(m.srcp), nowsize);
            }
        }
    }
}


